@@ -17,7 +17,7 @@ summary.sg<-function(object, ...)
    print(summary(degs))
    if("pp"%in%names(args))
    {
-	   l<-sg.edgelengths(object,args$pp)
+	   l<-edgelengths.sg(object,args$pp)
 	   cat("Edge length stats:\n")
 	   print(summary(l$d))
    }
