@@ -14,12 +14,16 @@ class Pp
 	int ntypes;
 	int tor;
 	int *toroidal;
-
+	double windowArea;
 
 	double (Pp::*dist)(int*, int*);
+	double (Pp::*weight)(int*, int*);
 	double distEuclidian(int*, int*);
 	double distPrecalculated(int*, int*);
+	double weightAll1(int *, int *);
+	double weightTrans(int *, int *);
 	std::vector<double> distTriangle;
+	std::vector<double> weightTriangle;
 	std::vector<double> * pdists;
 	std::vector<int> typevec;
 
@@ -48,9 +52,15 @@ public:
 
 
 	double getDist(int *, int *);
+	void   setDist(int *, int *, double d);
 	void   calcDists();
 	void   setDists(double *);
 
+	double getWeight(int *, int *);
+	void   setWeight(int *, int *, double d);
+	void   calcTransWeights();
+	void   setAllTransWeights(double );
+
 	int    getCluster(int *);
 	int    nsize(int*); // neighbours
 	void   setMass(int *, double *);
@@ -59,6 +69,7 @@ public:
 	double getMass2(int *);
 
 	int Empty(int *, int *, int *); //voronoi
+	int EmptyConstrained(int *, int *, int *, std::vector <int>  *);
 
 	void addNeighbour(int *i, int *j);
 	void removeNeighbour(int *i, int *j);
